import json
import logging
import random
# Global variables
from concurrent.futures.thread import ThreadPoolExecutor
from json import JSONDecodeError

import requests

from enrich_pkg.custom_call_exception import CustomCallException

mappings = {}
enrichment_returned_data_dict = {}
user_data = {}
enrich_targets = {}


def log_with_tabs(depth, string_output):
    logging_str = ''
    for _ in range(0, depth):
        logging_str += '\t'
    logging_str += string_output
    logging.info(logging_str)


def get_value_from_mapping(which_one, param):
    incoming_user_data = ''
    if which_one in mappings and param in mappings[which_one]:
        incoming_var_name = mappings[which_one][param]
        if incoming_var_name in user_data.keys():
            incoming_user_data = user_data[incoming_var_name]
        else:
            if '_1_app_'+incoming_var_name in user_data.keys():
                incoming_user_data = user_data['_1_app_'+incoming_var_name]
    return incoming_user_data


def get_payload(which_one):
    request_params = {}
    params = enrich_targets.get(which_one).get('params')
    for param in params:
        incoming_user_data = get_value_from_mapping(which_one, param)
        request_params[param] = incoming_user_data

    return request_params


def get_body(which_one):
    body_shell = enrich_targets.get(which_one).get('body').copy()
    for key in body_shell:
        incoming_user_data = get_value_from_mapping(which_one, key)
        incoming_user_data_raw = incoming_user_data
        if isinstance(incoming_user_data, str) and isinstance(body_shell.get(key), bool):
            incoming_user_data = (incoming_user_data.lower() == 'true')
        if incoming_user_data_raw != '':
            if body_shell.__getitem__(key) != '':
                next_entry = body_shell.__getitem__(key)
                for subkey in next_entry:
                    body_shell[key][subkey] = incoming_user_data
            else:
                body_shell[key] = incoming_user_data
    return body_shell


def call_specific_enrichment(which_one):
    url = enrich_targets.get(which_one).get('url')
    mechanism = enrich_targets.get(which_one).get('mechanism')
    r = {}
    try:

        if mechanism == 'GET':
            payload = get_payload(which_one)
            if payload != '':
                r = requests.get(url, params=payload)
            else:
                r = requests.get(url)
        elif mechanism == 'POST':
            body = get_body(which_one)
            # NOTE - have to do the json.dumps, and the header -
            # otherwise we get back an offset in the return, making it not json!!
            payload = json.dumps(body)
            headers = {"Content-type": "application/json"}
            r = requests.post(url, data=payload, headers=headers)

    except requests.exceptions.HTTPError as errh:
        enrichment_returned_data_dict[which_one] = dict(result='failure', data={'http_error': errh})
        raise CustomCallException(errh)
    except requests.exceptions.ConnectionError as errc:
        enrichment_returned_data_dict[which_one] = dict(result='failure', data={'connection_error': errc})
        raise CustomCallException(errc)
    except requests.exceptions.Timeout as errt:
        enrichment_returned_data_dict[which_one] = dict(result='failure', data={'timeout_error': errt})
        raise CustomCallException(errt)
    except requests.exceptions.RequestException as err:
        logging.error('ERROR calling %s [%s]' % (which_one, err.args))
        enrichment_returned_data_dict[which_one] = dict(result='failure', data={'other_error': err.args})
        raise CustomCallException(err)

    return {'r': r, 'mechanism': mechanism, 'url': url}


def call_enrichment(which_one, depth):
    log_with_tabs(depth, 'calling enrichment %s' % which_one)
    if which_one in enrich_targets.keys():
        r_dict = call_specific_enrichment(which_one)

        try:
            results = r_dict['r'].json()
            if isinstance(results, list):
                results = results[0]
            enrichment_returned_data_dict[which_one] = dict(result='success',
                                                            data=results)
        except JSONDecodeError as err:
            logging.error("Json decode error on %s to %s: %s" % (r_dict['mechanism'], r_dict['url'], err))
            enrichment_returned_data_dict[which_one] = dict(result='failure',
                                                            data=(r_dict['mechanism'], r_dict['url'], err))
            raise Exception((r_dict['mechanism'], r_dict['url'], err))
    else:
        logging.error('No such target for enrichment %s' % which_one)
        raise Exception('No such target for enrichment %s' % which_one)


def serial_enrich_with_failover(serial_steps, failover_steps, depth):
    log_with_tabs(depth, 'in serialEnrichWithFailover %s %s' % (serial_steps, failover_steps))
    # Serially run those in serialSteps.  If any exceptions arise, switch and call those in failoverSteps
    on_which_step = ''
    try:
        for on_which_step in serial_steps:
            call_enrichment(on_which_step, depth + 1)
    except CustomCallException:
        logging.error("Exception raised in serial steps %s, failed on %s. Calling failover steps %s" %
                      (serial_steps, on_which_step, failover_steps))
        try:
            for on_which_step in failover_steps:
                call_enrichment(on_which_step, depth + 1)
        except CustomCallException:
            logging.error('Failed on fail-over calls - enrichment data will be lost %s' % on_which_step)


def run_single_step(a_step, depth, ran=100):
    lower = a_step.get('stepBoundsLower', 0)
    upper = a_step.get('stepBoundsUpper', 100)
    what = a_step.get('what', 'missing')

    if lower <= ran <= upper:
        log_with_tabs(depth, 'What %s, LowerBound %s, Upperbound %s' % (what, lower, upper))

        if 'serialsteps' in a_step.keys():
            # failoversteps are optional, will show as 'missing' in method
            serial_enrich_with_failover(a_step.get('serialsteps'), a_step.get('failoversteps', 'missing'), depth + 1)
        elif what.find(';') < 0:
            # At the end of the path, call the enrichment
            try:
                call_enrichment(what, depth)
            except CustomCallException as cce:
                logging.error('Failed on enrichment step %s\n\t%s' % (what, cce.message.args[0].reason))
        else:
            zstep = a_step.get('steps')
            run_single_step(zstep, depth + 1, ran)


def run_steps(steptuple):
    ran = steptuple[0]
    step_map = steptuple[1]
    for step in step_map['steps']:
        run_single_step(step, 1, ran)


def decide_with_json(enrich_targets_provided, enrich_step_definitions, incoming_mappings, json_user_data):
    """
    enrich_step_definitions:  step definitions in json format, describing how to call which enrichment tool
    incoming_mappings: get dictionary describing how data is mapped from ID to enrichment, and from enrichment to ID
    json_user_data: json data representing data coming from Intelligent Decisioning

    returns: tuple, value_dictionary
    """
    global mappings, user_data, enrichment_returned_data_dict, enrich_targets
    mappings = incoming_mappings
    user_data = json_user_data
    enrichment_returned_data_dict = {}
    enrich_targets = enrich_targets_provided
    concurrent_steps = []

    for a_step in enrich_step_definitions['steps']:
        ran = random.randint(0, 100)
        if 'steps' in a_step:
            concurrent_steps.append((ran, a_step))

    with ThreadPoolExecutor(max_workers=3) as executor:
        executor.map(run_steps, concurrent_steps)

    return enrichment_returned_data_dict



import json

from enrichwrap import get_contents


def get_default_data():
    with open(get_contents('samples', 'sample_input.json')) as json_user_data_file:
        return json.load(json_user_data_file)


def get_data(val=None):
    if val is not None:
        return json.loads(val)
    return get_default_data()



from .listexternal import print_contents, get_xml_files, get_sample_files, get_contents
from .indata import get_default_data, get_data
from .read_mappings import get_mapping,get_all_mappings,get_biocatch_mappings,get_socure_mappings,get_biocatch_mappings
from .read_mappings import read_to_sas_mappings,get_payfoneverify_mappings
from .steps import get_steps
from .ccexception import CustomCallException
from .targets import set_default_targets, get_targets, get_target, update_target, set_enrich_targets
from .runsteps import decide_with_json
from .tosas import convert_to_sas
from .main import enrich



import logging

def get_sas_outval(valToFind, enrichtarget, enrichment_returned_data_dict):
    marchinglist = []
    data_level = {}
    for a_string in valToFind.split('/'):
        if a_string != '':
            marchinglist.append(a_string)
    if marchinglist[0] not in enrichment_returned_data_dict.get(enrichtarget).keys() and \
            'data' in enrichment_returned_data_dict.get(enrichtarget).keys():
        # Go down into Data subkey
        data_level = enrichment_returned_data_dict.get(enrichtarget).get('data')
        if type(data_level) is list and len(data_level) == 1:
            data_level = data_level[0]
        if marchinglist[0] not in data_level.keys() and 'results' in data_level.keys():
            data_level = data_level.get('results')
        for this_item in marchinglist:
            if this_item.find('[') > 0 and this_item.find(']') > 0:
                main_part = this_item[0:this_item.find('[')]
                list_index = int(this_item[this_item.find('[')+1:this_item.rfind(']')])
                if type(data_level) is dict and main_part in data_level.keys() and type(data_level.get(main_part)) is list and len(data_level.get(main_part)) > list_index and this_item != marchinglist[len(marchinglist)-1]:
                    data_level = data_level.get(main_part)[list_index]
                    continue

                if type(data_level) is list and this_item == marchinglist[len(marchinglist)-1]:
                    if list_index+1 < len(data_level):
                        return data_level[list_index]
                    else:
                        raise Exception('not found')

                if main_part in data_level.keys():
                    data_list = data_level.get(main_part)
                    if len(data_list) > list_index:
                        return data_list[list_index]
                    else:
                        raise Exception('not found')
                else:
                    raise Exception('not found')
            if type(data_level) is list and this_item == marchinglist[len(marchinglist)-1]:
                return data_level
            if this_item not in data_level.keys():
                # for the case of bokumaa, the data can come back without the hierarchy the xml has
                if marchinglist[len(marchinglist)-1] in data_level.keys():
                    return data_level.get(marchinglist[len(marchinglist)-1])
                raise Exception('not found')
            data_level = data_level.get(this_item)
    return data_level


def convert_to_sas(enrichment_returned_data_dict, mappings):
    out_dictionary = {}
    for enrich in enrichment_returned_data_dict.keys():
        if (enrich+'_out') in mappings.keys():
            sas_mappings = mappings.get(enrich + "_out").copy()
            logging.info('-------------------------------------------------------------------')
            logging.info('For enrichment %s' % enrich)
            logging.info('sas_mappings: %s' % sas_mappings)
            logging.info('enriched_data %s' % enrichment_returned_data_dict.get(enrich))
            print(enrichment_returned_data_dict.get(enrich))
            for out_sas_key in sas_mappings.keys():
                try:
                    val2 = get_sas_outval(sas_mappings.get(out_sas_key), enrich, enrichment_returned_data_dict)
                    out_dictionary[out_sas_key] = val2
                except:
                    logging.info('value not found for %s' % out_sas_key)
    return out_dictionary




class CustomCallException(Exception):
    """Exception raised for errors in connection to third party enrichment

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message="Salary is not in (5000, 15000) range"):
        self.message = message
        super().__init__(self.message)



import os
import xml.dom.minidom

mapping_common_to_SAS = {}


def get_mappings_in_out(
        in_enrichname='biocatch_getscore', in_file_name='biocatch_getscore_payload.xml', in_subtype='getScore',
        out_enrichname='biocatch_out', out_file_name='biocatch_to_sas.xml', out_mapperId='biocatchRespToOde',
        basedir=None):

    if basedir is None:
        basedir = os.path.dirname(__file__) + os.path.sep + 'xml' + os.path.sep

    if in_subtype not in get_mapping(in_file_name, basedir):
        print('stop here!')
    mapping_in = get_mapping(in_file_name, basedir)[in_subtype]

    if out_mapperId not in read_to_sas_mappings(out_file_name):
        print('stop there')
    mapping_out = read_to_sas_mappings(out_file_name)[out_mapperId]

    return {in_enrichname: mapping_in, out_enrichname: mapping_out}


def get_subtype(is_common_mapping, elem):
    sub_type = 'payload'
    if not is_common_mapping:
        attr = ''
        node_i_am_on = elem
        while node_i_am_on.parentNode != '' and attr != 'message':
            attr = node_i_am_on.parentNode.tagName
            node_i_am_on = node_i_am_on.parentNode
        sub_type = node_i_am_on.getAttribute('subType')
    return sub_type


def get_mapper_id(is_common_mapping, elem):
    sub_type = ''
    if not is_common_mapping:
        attr = ''
        node_i_am_on = elem
        while node_i_am_on.parentNode != '' and attr != 'message':
            attr = node_i_am_on.parentNode.tagName
            node_i_am_on = node_i_am_on.parentNode
        sub_type = node_i_am_on.getAttribute('mapperId')
    return sub_type


def read_to_sas_mappings(infile, basedir=None):
    if basedir is None:
        basedir = os.path.dirname(__file__) + os.path.sep + 'xml' + os.path.sep

    source_to_target = {}
    dom_tree = xml.dom.minidom.parse(basedir + os.path.sep + infile)
    source_fields = dom_tree.documentElement.getElementsByTagName('srcField')
    previous_check_for_array0 = ''
    previous_target = ''
    for elem in source_fields:
        mapper_id = get_mapper_id(False, elem)
        if mapper_id not in source_to_target.keys():
            source_to_target[mapper_id] = {}

        source_field = elem.getAttribute('name')
        target_field = elem.parentNode.getAttribute('name')
        if target_field[0:1] == '/':
            target_field = target_field[1:]

        source_to_target[mapper_id][target_field] = source_field

        if source_field.find('[') > 0 and source_field.find(']') > 0:
            source_up_to_bracket = source_field[0: source_field.find('[')]
            previous_is_missing_bracket = previous_check_for_array0.find('[') < 0
            has_prev_str = source_field.find(previous_check_for_array0) >= 0
            zeroth_element = has_prev_str and source_field[source_field.find(previous_check_for_array0) + len(previous_check_for_array0)] == '['
            if previous_is_missing_bracket:
                if has_prev_str and zeroth_element:
                    source_to_target[mapper_id][previous_target] = previous_check_for_array0 + '[0]'
                elif previous_check_for_array0.startswith(source_up_to_bracket):
                    if previous_check_for_array0[previous_check_for_array0.rindex('/')+1] == source_field[source_field.rindex('/')+1]:
                        endbit = previous_check_for_array0[len(source_up_to_bracket):]
                        newval = source_up_to_bracket + '[0]' + endbit
                        source_to_target[mapper_id][previous_target] = newval

        previous_check_for_array0 = source_field
        previous_target = target_field

    return source_to_target


def read_default_mappings(xmldir, infile, isCommonMapping):
    global mapping_common_to_SAS

    if xmldir is None:
        xmldir = os.path.dirname(__file__) + os.path.sep + 'xml' + os.path.sep

    source_to_target = {}
    dom_tree = xml.dom.minidom.parse(xmldir + os.path.sep + infile)
    source_fields = dom_tree.documentElement.getElementsByTagName('srcField')
    for elem in source_fields:
        # Get subType for context I am in
        sub_type = get_subtype(isCommonMapping, elem)
        if sub_type not in source_to_target.keys():
            source_to_target[sub_type] = {}

        source_field = elem.getAttribute('name')
        if source_field[0:1] == '/':
            source_field = source_field[1:]
        target_field = elem.parentNode.getAttribute('name')
        if target_field[0:1] == '/':
            target_field = target_field[1:]
        if not isCommonMapping:
            if source_field in mapping_common_to_SAS.keys():
                source_field = mapping_common_to_SAS.get(source_field)
            else:
                if source_field.find('/') > 0:
                    source_field = source_field[source_field.rindex('/') + 1:]
                    if source_field in mapping_common_to_SAS.keys():
                        source_field = mapping_common_to_SAS.get(source_field)

        if isCommonMapping:
            source_to_target[sub_type][source_field] = target_field
        else:
            source_to_target[sub_type][target_field] = source_field

    return source_to_target


def get_mapping (which_file, xmldir=None, include_sas=False):
    global mapping_common_to_SAS

    if len(mapping_common_to_SAS.keys()) == 0:
        mapping_common_to_SAS = read_default_mappings(xmldir, 'common_to_sas.xml', True)

    return read_default_mappings(xmldir, which_file, include_sas)


def get_payfoneverify_mappings():
    return get_mappings_in_out('payfoneverify', 'payfone_payload.xml', 'verify',
                               'payfoneverify_out', 'payfone_to_sas.xml', 'payfoneverifyRespToOde')


def get_payfonetrust_mappings():
    return get_mappings_in_out('payfonetrust', 'payfone_payload.xml', 'trust',
                               'payfonetrust_out', 'payfone_to_sas.xml', 'payfonetrustRespToOde')


def get_bokugpir_mappings():
    return get_mappings_in_out('boku_gpir', 'boku_gpir_payload.xml', 'getPhoneIdentificationResult',
                               'boku_gpir_out', 'boku_gpir_to_sas.xml', 'bokugpirRespToOde')


def get_bokumaa_mappings():
    return get_mappings_in_out('boku_maa', 'boku_maa_payload.xml', 'matchAndAttributes',
                               'boku_maa_out', 'boku_maa_to_sas.xml', 'bokumaaRespToOde')


def get_socure_mappings():
    return get_mappings_in_out('socure', 'socure_payload.xml', 'payload',
                               'socure_out', 'socure_to_sas.xml', 'socureRespToOde')


def get_datavisor_mappings():
    return get_mappings_in_out('datavisor', 'datavisor_payload.xml', 'payload',
                               'datavisor_out', 'datavisor_to_sas.xml', 'datavisorRespToOde')


def get_giact_mappings():
    return get_mappings_in_out('giact', 'giact_payload.xml', 'payload',
                               'giact_out', 'giact_to_sas.xml', 'giactRespToOde')


def get_iovation_mappings():
    return get_mappings_in_out('iovation', 'iovation_payload.xml', 'payload',
                               'iovation_out', 'iovation_to_sas.xml', 'iovationRespToOde')


def get_biocatch_mappings():
    return get_mappings_in_out('biocatch', 'biocatch_getscore_payload.xml', 'getScore',
                               'biocatch_out', 'biocatch_to_sas.xml', 'biocatchRespToOde')


def get_all_mappings():
    global mapping_common_to_SAS

    mapping_payfone_verify = get_payfoneverify_mappings()
    mapping_payfone_trust = get_payfonetrust_mappings()
    mapping_boku_gpir = get_bokugpir_mappings()
    mapping_boku_maa = get_bokumaa_mappings()
    mapping_socure = get_socure_mappings()
    mapping_datavisor = get_datavisor_mappings()
    mapping_giact = get_giact_mappings()
    mapping_iovation = get_iovation_mappings()
    mapping_biocatch = get_biocatch_mappings()

    retmap = {}
    retmap.update(mapping_payfone_verify)
    retmap.update(mapping_payfone_trust)
    retmap.update(mapping_boku_gpir)
    retmap.update(mapping_boku_maa)
    retmap.update(mapping_socure)
    retmap.update(mapping_datavisor)
    retmap.update(mapping_giact)
    retmap.update(mapping_iovation)
    retmap.update(mapping_biocatch)
    return retmap



import glob
import os
from pathlib import Path


def print_contents ():
    starting_dir = os.path.dirname(__file__)
    print('__file__ contents: ')
    list_main = glob.glob(starting_dir + '*.*')
    for i in list_main:
        print('\t' + i[len(starting_dir):])
    print()

    print('__file__' + os.path.sep + 'samples contents: ')
    samples = starting_dir + os.path.sep +  'samples' + os.path.sep
    list_samples = glob.glob(samples + '*.*')
    for i in list_samples:
        print('\t' + i[len(samples):])
    print()

    print('__file__' + os.path.sep + 'xml contents: ')
    xmls = starting_dir + os.path.sep + 'xml' + os.path.sep
    list_xml = glob.glob(xmls + '*.*')
    for i in list_xml:
        print('\t' + i[len(xmls):])


def get_xml_files():
    starting_dir = os.path.dirname(__file__)
    xmls = starting_dir + os.path.sep + 'xml' + os.path.sep
    list_xml = glob.glob(xmls + '*.*')
    list_clean = []
    for i in list_xml:
        list_clean.append(i[len(xmls):])
    return list_clean


def get_sample_files():
    starting_dir = os.path.dirname(__file__)
    samples = starting_dir + os.path.sep +  'samples' + os.path.sep
    list_samples = glob.glob(samples + '*.*')
    list_clean = []
    for i in list_samples:
        list_clean.append(i[len(samples):])
    return list_clean


def get_contents(dirname, filename):
    starting_dir = os.path.dirname(__file__)
    file_loc = starting_dir + os.path.sep + dirname + os.path.sep + filename
    if os.path.isfile(file_loc):
        return file_loc
    return None


if __name__ == "__main__":
    print_contents()



import logging
import os
import sys

from enrichwrap import get_data, get_steps, get_all_mappings, set_default_targets, decide_with_json, \
    convert_to_sas


def enrich(step_string, enrich_targets=None, incoming_data=None):
    logname = os.path.dirname(__file__) + os.path.sep + 'pythonenrich.log'
    logging.basicConfig(filename=logname,
                        format='[%(asctime)s] %(levelname)s:%(message)s',
                        datefmt='%m/%d/%Y %H:%M:%S',
                        level=logging.INFO)

    step_dictionary = get_steps(step_string)
    if isinstance(incoming_data, str):
        incoming_data = get_data(incoming_data)

    if enrich_targets is None:
        enrich_targets = set_default_targets()

    mappings = get_all_mappings()

    resulting_data = decide_with_json(enrich_targets, step_dictionary, mappings, incoming_data)
    sas_values = convert_to_sas(resulting_data, mappings)

    #print(json.dumps(resulting_data, indent=4))
    #print(json.dumps(sas_values, indent=4))
    return {"tool_data": resulting_data, "sas_data": sas_values}


if __name__ == '__main__':
    # setup_default_enrich_targets

    # Get default user data, as a starting point
    user_data = None

    # Get default step string of None as a starting point
    stepString = None
    if len(sys.argv) > 1:
        stepString = sys.argv[1]

    if len(sys.argv) > 2:
        user_data = sys.argv[2]

    enrich(stepString, user_data)




import pathlib

from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

here = pathlib.Path(__file__).parent.resolve()

setup(
    # This is the name of your project. The first time you publish this
    # package, this name will be registered for you. It will determine how
    # users can install this project, e.g.:
    #
    # $ pip install sampleproject
    #
    # And where it will live on PyPI: https://pypi.org/project/file-parsing-sample/
    #
    # There are some restrictions on what makes a valid project name
    # specification here:
    # https://packaging.python.org/specifications/core-metadata/#name
    name="enrichwrap",

    # Versions should comply with PEP 440:
    # https://www.python.org/dev/peps/pep-0440/
    #
    # For a discussion on single-sourcing the version across setup.py and the
    # project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.0.27',

    # This is a one-line description or tagline of what your project does. This
    # corresponds to the "Summary" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#summary
    description="A small package testing calling out to third party enrichment",  # Optional

    # This is an optional longer description of your project that represents
    # the body of text which users will see when they visit PyPI.
    #
    # Often, this is the same as your README, so you can just read it in from
    # that file directly (as we have already done above)
    #
    # This field corresponds to the "Description" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#description-optional
    long_description=long_description,  # Optional

    # Denotes that our long_description is in Markdown; valid values are
    # text/plain, text/x-rst, and text/markdown
    #
    # Optional if long_description is written in reStructuredText (rst) but
    # required for plain-text or Markdown; if unspecified, "applications should
    # attempt to render [the long_description] as text/x-rst; charset=UTF-8 and
    # fall back to text/plain if it is not valid rst" (see link below)
    #
    # This field corresponds to the "Description-Content-Type" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#description-content-type-optional
    long_description_content_type='text/markdown',  # Optional (see note above)

    # Classifiers help users find your project by categorizing it.
    #
    # For a list of valid classifiers, see https://pypi.org/classifiers/
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate you support Python 3. These classifiers are *not*
        # checked by 'pip install'. See instead 'python_requires' below.
        "Programming Language :: Python :: 3",

        "Operating System :: OS Independent",

        'Topic :: Text Processing :: Linguistic',
    ],
    keywords='enrichment',
    url='http://github.com/suhens/enrichwrap',

    # This should be your name or the name of the organization which owns the project.
    author='Sue Hannum',

    # This should be a valid email address corresponding to the author listed above.
    author_email='Sue.Hannum@sas.com',
    license_files=('LICENSE',),

    # This should be a valid link to your project's main homepage.
    #
    # This field corresponds to the "Home-Page" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#home-page-optional
    # url='https://github.com/pypa/sampleproject',  # Optional
    # project_urls={
    #    "Bug Tracker": "https://github.com/pypa/sampleproject/issues",
    # },

    # When your source code is in a subdirectory under the project root, e.g.
    # `src/`, it is necessary to specify the `package_dir` argument.
    # package_dir={"": "src"}, # Optional

    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    # packages=find_packages(where="src"), # Required
    packages=['enrichwrap'],

    # Specify which Python versions you support. In contrast to the
    # 'Programming Language' classifiers above, 'pip install' will check this
    # and refuse to install the project if the version does not match. See
    # https://packaging.python.org/guides/distributing-packages-using-setuptools/#python-requires
    python_requires=">=3.6",

    # This field lists other packages that your project depends on to run.
    # Any package you put here will be installed by pip when your project is
    # installed, so they must be valid existing projects.
    #
    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[
        'requests',
        'markdown',
    ],  # Optional

    include_package_data=True,
    zip_safe=False,
    test_suite='nose.collector',
    tests_require=['nose'],
    scripts=['bin/enrichwrap'],
    entry_points={
        'console_scripts': [
            'enrichmain=enrichwrap.command_line:main',
        ],
    }
    # If there are data files included in your packages that need to be
    # installed, specify them here.
    # package_data={  # Optional
    #     'sample': ['package_data.dat'],
    # },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/distutils/setupscript.html#installing-additional-files
    #
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[('my_data', ['data/data_file'])],  # Optional

)



import logging

enrich_targets = {}


def set_default_targets():
    global enrich_targets
    enrich_targets = {
        'socure': {
            'url': 'http://localhost:8100/socure/items',
            'body': {
                'modules': [
                    'fraud', 'phonerisk', 'addressrisk', 'emailrisk', 'kyc', 'social', 'watchliststandard', 'alertlist'
                ],
                'firstName': '',
                'surName': '',
                'email': '',
                'dob': '',
                'physicalAddress': '',
                'physicalAddress2': '',
                'city': '',
                'state': '',
                'country': '',
                'zip': '',
                'mobileNumber': '',
                'nationalId': '',
                'driverLicense': '',
                'driverLicenseState': '',
            },
            'mechanism': 'POST'
        },
        # TODO - need to check on payfone site for details if different than ID360/boss
        'payfonetrust': {
            'url': 'http://localhost:8100/payfone/trust/v2',
            'body': {
                'requestId': '',
                'details': '',
                'consentStatus': '',
                'phoneNumber': '',
            },
            'mechanism': 'POST'
        },
        'payfoneverify': {
            'url': 'http://localhost:8100/payfone/identity/verify/v2',
            'body': {
                'requestId': '',
                'payfoneAlias': '',
                'firstName': '',
                'lastName': '',
                'address': '',
                'extendedAddress': '',
                'city': '',
                'region': '',
                'postalCode': '',
                'lastVerified': '',
                'details': False,
                'phoneUpdate': False,
            },
            'mechanism': 'POST'
        },
        'threatmetrix': {
            'url': 'http://localhost:8100/threadmetrix',
            'params': [],
            'mechanism': 'GET'
        },
        #    'giact': 'http://localhost:8100/giact/items',
        'biocatch': {
            'url': 'http://localhost:8100/biocatch/items',
            'params': [],
            'mechanism': 'GET'
        },
        'bokugpir': {
            # getPhoneIdentificationResult
            'url': 'http://localhost:8100/boku/retrieve-evurl/items',
            'body': {
                'merchantId':'',
                'correlationId': '',
                'associationKey': '',
                'match': {
                    'consumerMdn': ''
                }
            },
            'mechanism': 'POST'
        },
        'bokumaa': {
            'url': 'http://localhost:8100/boku/match/items',
            'body': {
                'merchantId': '',
                'subMerchantId': '',
                'consentId': '',
                'consentTimeStamp': '',
                'msisdn': '',
                'correlationId': '',
                'firstName': '',
                'lastName': '',
                'address1': '',
                'address2': '',
                'city': '',
                'state': '',
                'postalCode': '',
                'countryCode': '',
                'nationalId': '',
                'dateOfBirth': ''
            },
            'mechanism': 'POST'
        }
    }

    return enrich_targets


def update_target(which_enrich, map_new_data):
    # Validate that I have what I need
    if 'url' not in map_new_data.keys() or 'mechanism' not in map_new_data.keys():
        logging.error('MISSING values in updating enrich target - ignoring.  '
                      'Need "url" and "mechanism" at the least [%s]'%map_new_data)
        return
    mechanism = map_new_data.get('mechanism')
    if mechanism.lower() == 'post' and 'body' not in map_new_data.keys():
        logging.error('If POST is the mechanism, then body must be provided [%s]' % map_new_data)
        return
    elif mechanism.lower() == 'get' and 'params' not in map_new_data.keys():
        logging.error('If GET is the mechanism, then params must be provided [%s]' % map_new_data)
        return

    enrich_targets[which_enrich] = map_new_data


def get_target(which_enrich):
    if len(enrich_targets) == 0:
        set_default_targets()
    return enrich_targets[which_enrich]


def set_enrich_targets(map):
    global enrich_targets
    enrich_targets = map
    return enrich_targets


def get_targets():
    return enrich_targets



import json
from enrichwrap import get_contents


def get_steps(step_string=None):
    """
    args: The list of arguments as would be passed in from the command line or the IDE's run.
            This list starts after the 0th element (the python file name)

    returns: dictionary, describing the steps aka how the enrichment tools will be called, and failovers and percentages
    """
    json_dictionary = {}
    if step_string is not None:
        print('Parse input of %s' % step_string)
        json_dictionary = parse_incoming_string(step_string)

    # If the string is not specified, or cannot be parsed, then use the file as a backup
    if len(json_dictionary.keys()) == 0:
        with open(get_contents('samples', 'input.json')) as json_file:
            step_dictionary = json.load(json_file)
    else:
        step_dictionary = json_dictionary

    return step_dictionary


def parse_incoming_string(incoming_string):
    json_dictionary = {}
    sample_list = []
    split_str = incoming_string.split(',')
    step_string = ''
    astr = ''
    for astr in split_str:
        if astr.find(':') > 0:
            if len(step_string) > 0:
                step_string += ','
            step_string += astr
        elif step_string != '':
            sample_list.append(get_step_definition(step_string))
            step_string = ''
        else:
            step_string = astr
    sample_list.append(get_step_definition(astr))

    json_dictionary["steps"] = sample_list
    #print(json.dumps(json_dictionary))
    return json_dictionary


def get_failover(step_string):
    if step_string.find(';') < 0 and step_string.find('|') < 0:
        return step_string
    steps = {}
    main_step_str = step_string
    failover_step_str = ''
    if step_string.find('|') > 0:
        main_step_str = step_string[0: step_string.find('|')]
        failover_step_str = step_string[step_string.find('|')+1:]
    steps['what'] = main_step_str
    steps['failoverWhat'] = failover_step_str
    my_list = []
    for astr in main_step_str.split(';'):
        my_list.append(astr)
    steps['serialsteps'] = my_list
    failover_list = []
    for astr in failover_step_str.split(';'):
        failover_list.append(astr)
    steps['failoversteps'] = failover_list
    return steps


def get_step_definition(step_string):
    step_dictionary = {'what': step_string}
    step_list = []
    # If step_string contains a comma, then there are more than one steps to create, with the bounds as defined
    for astr in step_string.split(','):
        if astr.find(':') > 0:
            enrich_str = astr[0:astr.find(':')]
            percentage_bit = astr[astr.find(':')+1:]
            lower_bound = percentage_bit[0: percentage_bit.find('-')]
            upper_bound = percentage_bit[percentage_bit.find('-')+1:]
            this_step = {'stepBoundsLower': int(lower_bound), 'stepBoundsUpper': int(upper_bound), 'what': enrich_str}
            if enrich_str.find('|') > 0 or enrich_str.find(';') > 0:
                this_step['steps'] = get_failover(enrich_str)
            step_list.append(this_step)
        else:
            this_step = {'what': astr}
            if astr.find('|') > 0 or astr.find(';') > 0:
                this_step['steps'] = get_failover(astr)
            step_list.append(this_step)
    step_dictionary['steps'] = step_list
    return step_dictionary

